'############################################################
' Bascom AVR Test
' Powertip PH9867C CSTN Color Display
' Tapeten-Muster Demonstration
' Display uses the ST7626 Display Controler
' Tested on a ATMega8 set to 1Mhz internal osc
'############################################################

$crystal = 1000000
$regfile = "m8def.dat"

Dim I As Byte , J As Byte , N As Byte , Rcc As Byte

Lcd_a0 Alias Portc.5
Lcd_cs Alias Portc.4
Lcd_reset Alias Portc.3
Lcd_wr Alias Portc.2
Lcd_data Alias Portd

Config Lcd_a0 = Output
Config Lcd_cs = Output
Config Lcd_reset = Output
Config Lcd_wr = Output
Config Lcd_data = Output


Const Cmd = 0
Const Dat = 1

Declare Sub Lcd_write(byval L_cmd As Byte , Byval L_dat As Byte)
Declare Sub Lcd_init()
Declare Sub Lcd_loadpaint()
Declare Sub Lcd_loadeeprom()
Declare Sub Lcd_test()

' The Main Loop goes here
' -----------------------
Lcd_init
waitms 500
Do
Lcd_test
waitms 50
Loop
'------------------------
End

Sub Lcd_init()                                              '
   Set Lcd_cs                                               ' set display inactive
   Reset Lcd_reset
   Waitms 10
   Set Lcd_reset                                            ' Reset the LCD controler
   Waitms 50                                                ' wait a little for the LCD to be ready

   Lcd_write Cmd , &H30                                     ' Ext = 0
   Lcd_write Cmd , &H04                                     ' Sleep In / Out Preparation
   Lcd_write Dat , &H3E                                     ' Sleep In / Out Sequencing
   Lcd_write Cmd , &H31                                     ' Ext = 1
   Lcd_write Cmd , &HF4                                     ' Internal Initialize Preparation
   Lcd_write Dat , &H58                                     ' Internal Initialize Sequencing
   Lcd_write Cmd , &H30                                     ' Ext = 0
   Lcd_write Cmd , &H94                                     ' Sleep Out
   Lcd_write Cmd , &HD1                                     ' Internal Osc On
   Lcd_write Cmd , &HCA                                     ' Display Control
   Lcd_write Dat , &H00                                     ' Cl Divisions Ratio
   Lcd_write Dat , &H10                                     ' Duty Setting( = 68)
   Lcd_write Dat , &H00                                     ' N -line Inverse -set Value
   Lcd_write Cmd , &H31                                     ' Ext = 1
   Lcd_write Cmd , &H32                                     ' Analog Setting
   Lcd_write Dat , &H00                                     ' Osc Freqency Adjustment
   Lcd_write Dat , &H02                                     ' Booster Efficiency Setting Is Level3
   Lcd_write Dat , &H03                                     ' Bias Setting( = 1 / 9)
   Lcd_write Dat , &H04                                     ' Booster Is X5
   Lcd_write Cmd , &H30                                     ' Ext = 0
   Lcd_write Cmd , &H81                                     ' Electronic Volume Control
   Lcd_write Dat , &H2D                                     ' Ev : Vop[5 : 0]_6bit
   Lcd_write Dat , &H02                                     ' Ev : Vop[8 : 6]_3bit
                                                             ' Vop Is 10.52v Under This Condition For Example
   Lcd_write Cmd , &H20                                     ' Power Control
   Lcd_write Dat , &H0B                                     ' B / F / R = On / On / On
   Waitms 50                                                ' Delay 50ms
   Lcd_loadeeprom                                           ' Load EEPROM example program (refer page 68)
   Lcd_loadpaint                                            ' Load Gamma Table Parameter (refer page 61)
   Lcd_write Cmd , &H30                                     ' Ext = 0
   Lcd_write Cmd , &HA7                                     ' Inverse Display
   Lcd_write Cmd , &HBB                                     ' Com Scan Direction
   Lcd_write Dat , &H01                                     ' 0~33 / 67~34
   Lcd_write Cmd , &HBC                                     ' dat Scan Direction
   Lcd_write Dat , &H00                                     ' Page / Column Address Setting
   Lcd_write Dat , &H00                                     ' RGB arrangement (0:RGB 1:BGR)
   Lcd_write Dat , &H01                                     ' Gray-scale setup ( 64-gray: 01H)
   Lcd_write Cmd , &H75                                     ' Page address set
   Lcd_write Dat , &H00                                     ' From page address 0
   Lcd_write Dat , 67                                       ' to page address 67
   Lcd_write Cmd , &H15                                     ' Column address set
   Lcd_write Dat , &H00                                     ' From column address 0
   Lcd_write Dat , 97                                       ' to column address 97
   Lcd_write Cmd , &HAF                                     ' Display On
   Lcd_write Cmd , &H30                                     ' Ext = 0
End Sub

Sub Lcd_loadpaint()                                         ' Setup the gamma correction tables
   Lcd_write Cmd , &H31
   Lcd_write Cmd , &H20
   Restore Lut1
   For N = 0 To 15
      Read Rcc
      Lcd_write Dat , Rcc
   Next N
      Lcd_write Cmd , &H21
   Restore Lut2
   For N = 0 To 15
      Read Rcc
         Lcd_write Dat , Rcc
   Next N
      Lcd_write Cmd , &H22
   Restore Lut3
   For N = 0 To 15
      Read Rcc
         Lcd_write Dat , Rcc
   Next N
      Lcd_write Cmd , &H23
   Restore Lut4
   For N = 0 To 15
      Read Rcc
         Lcd_write Dat , Rcc
   Next N
      Lcd_write Cmd , &H30
End Sub

Sub Lcd_test()

   Dim Ti As Byte
      Ti = Rnd(255)
      Lcd_write Cmd , &H30
      Lcd_write Cmd , &H15
      Lcd_write Dat , 0
      Lcd_write Dat , 97
      Lcd_write Cmd , &H75
      Lcd_write Dat , 0
      Lcd_write Dat , 67
      Lcd_write Cmd , &H5C
    For J = 0 To 67
      Rotate Ti , Left , J
      For I = 0 To 97
          Rotate Ti , Left , I
          Lcd_write Dat , Ti
          Lcd_write Dat , Ti
      Next I
    Next J
End Sub


Sub Lcd_loadeeprom()                                        ' Load the EEPROM contents into the LCD registers
   Lcd_write Cmd , &H31
   Lcd_write Cmd , &HCD
   Lcd_write Dat , &H00
   Waitms 50
   Lcd_write Cmd , &HFD
   Waitms 50
   Lcd_write Cmd , &HCC
   Lcd_write Cmd , &H30
End Sub


Sub Lcd_write(byval L_cmd As Byte , Byval L_dat As Byte)
   Lcd_a0 = L_cmd
   Lcd_data = L_dat
   Reset Lcd_cs
   Reset Lcd_wr
   ' YOU MAYBE NEED A DELAY HERE FOR HIGHER CPU SPEEDS
   Set Lcd_wr
   Set Lcd_cs
End Sub


' Gamma correction table
Lut1:
Data 0 , 1 , 4 , 6 , 8 , 11 , 12 , 13 , 14 , 16 , 17 , 18 , 19 , 21 , 24 , 28
Lut2:
Data 0 , 2 , 5 , 7 , 9 , 11 , 12 , 13 , 14 , 16 , 17 , 18 , 19 , 22 , 25 , 29
Lut3:
Data 0 , 3 , 6 , 8 , 10 , 12 , 13 , 14 , 15 , 16 , 17 , 19 , 20 , 23 , 26 , 30
Lut4:
Data 0 , 4 , 7 , 9 , 11 , 12 , 13 , 14 , 15 , 16 , 17 , 19 , 21 , 24 , 27 , 31